﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import axios, { AxiosRequestConfig } from "axios";
import { QueryClient, useMutation, useQuery, useQueryClient } from "react-query";
import { useAuth } from "../lib/auth/AuthContext";
import { API_URL, reportAxiosError } from "../utils/RequestUtils";
import { LMComment } from "./useComments";

/**
 * Interface representing a landmark object
 */
export interface Landmark {
    /**
     * The id of the landmark.
     */
    id?: string | null,
    /**
     * The rating of the landmark.
     */
    rating?: number | null,
    /**
     * The id of the user who created the landmark.
     */
    user?: string | null,
    /**
     * The x coordinate (or longitude) of the landmark's location.
     */
    longitude?: number | null,
    /**
     * The y coordinate (or latitude) of the landmark's location.
     */
    latitude?: number | null, 
    /**
     * The landmark's title.
     */
    title?: string | null,
    /**
     * The landmark's description.
     */
    description?: string | null,
    /**
     * User [comments]{@link LMComment} associated with this landmark.
     */
    comments?: LMComment[] | null,
    /**
     * [Photos]{@link LMPhoto} associated with this landmark.
     */
     photos?: LMPhoto[] | null,
    /**
     * An integer representing the type of this landmark.
     */
    landmark_type?: number | null, // for working with existing database schema, should be changed
    /**
     * A Date object representing when this landmark was created.
     */
    time?: Date | null,
    //TODO: add floor property
}

export interface  LMPhoto {
    id: string
    landmark: string,
    image_b64: string
    height: number,
    width: number
}

export interface UseLandmarkOptions {
    userId?: string
    landmarkId?: string
    userLMPairing?: {userId?: string, landmarkId?: string}
    changingPassword?: boolean
}


export const useAllLandmarks = () => {
    const {sendRequestAsync} = useAuth()
    const queryClient: QueryClient = useQueryClient()

    const fetchAllLandmarks = async () => {
        const response = await sendRequestAsync({
            url: `/api/landmarks`,
            method: "GET",
        })
        return response.data
    }

    const {data: landmarks, status, error} = useQuery(
        "allLandmarks",
        fetchAllLandmarks,
        {
            retry: false,
            refetchOnWindowFocus: false,
            staleTime: Infinity,
            cacheTime: Infinity,
            onSuccess: () => {
                queryClient.invalidateQueries("allLandmarks")
            }
        }
    )

    return {
        landmarks,
        status,
        error
    }
}

export const useLandmarks = (options?: UseLandmarkOptions) => {
    const {sendRequestAsync} = useAuth()

    /**
     * The local instance of the [react-query QueryClient]{@link https://react-query.tanstack.com/reference/QueryClient#_top}.
     * @memberOf useLandmarks
     */
    let queryClient: QueryClient;

    try {
        queryClient = useQueryClient()
    } catch (error) {
        console.error("[LandmarkData]: Something went wrong when retrieving query client: " + error)
    }

    /**
     * The callback responsible for retrieving {@link Landmark} from the API, used by the [react-query useQuery]{@link https://react-query.tanstack.com/reference/useQuery#_top} hook.
     * * @memberOf useLandmarks
     */
    const getLandmarks = async () => {
        let url = `/api/landmarks/`   
        const config: AxiosRequestConfig = {
            method: 'GET',
            url: url,
        } 

        const response = await sendRequestAsync(config)
        console.log("retrieved landmarks. count: " + response.data.length)
        return response.data
    }

    /**
     * The callback responsible for retrieving photos for a {@link Landmark}, used by the [react-query useQuery]{@link https://react-query.tanstack.com/reference/useQuery#_top} hook.
     * * @memberOf useLandmarks
     */
    const getLandmark = async (landmarkId?: string) => {
        if (landmarkId) {
            const config: AxiosRequestConfig = {
                method: 'GET',
                url: `/api/landmark/${landmarkId}`,
            }
    
            const response = await sendRequestAsync(config)
            return response.data
        }
        
        // }
        // else {
        //     return [];
        // }
    }

    const addLandmarkPhoto = async (photo: LMPhoto) => {
        if (photo) {
            const config: AxiosRequestConfig = {
                method: 'POST',
                url: `/api/landmark/photos/`,
                data: photo,
            }
    
            const response = await sendRequestAsync(config)
            return response.data
        }
    }

    const deleteLandmarkPhoto = async (photoId: string) => {
        if (photoId) {   
            const config: AxiosRequestConfig = {
                method: 'DELETE',
                url: `/api/landmark/photos/${photoId}`,
            }
    
            const response = await sendRequestAsync(config)
            return response.data
        }
        
        // }
        // else {
        //     return [];
        // }
    }

    const checkIfRatedByUser = async (userLMPairing: {userId?: string, landmarkId?: string}) => {
        if (options?.userLMPairing) {
            if (userLMPairing?.landmarkId && userLMPairing?.userId) {
                const config: AxiosRequestConfig = {
                    method: 'GET',
                    url: `/api/landmark/check-rate-pairing/?landmark=${options?.userLMPairing?.landmarkId}&user=${options.userLMPairing.userId}`,
                } 
        
                const response = await sendRequestAsync(config)
                return response.data
            }
            else {
                console.warn("[LandmarkData]: Can't check landmark rating pairing. Given id values must not be null.")
            }
        }
    }

     /**
     * The callback responsible for adding a new {@link Landmark} to the server, used by the [react-query useMutation]{@link https://react-query.tanstack.com/reference/useMutation#_top} hook.
     * * @memberOf useLandmarks
     */
    const createLandmark = async (data: {landmarkValue: Landmark | undefined, photos?: LMPhoto[]}): Promise<Landmark | undefined> => {
        if (data.landmarkValue) {
            const config: AxiosRequestConfig = {
                method: 'POST',
                data: {
                    landmark: data.landmarkValue,
                    photos: data.photos
                },
                url: `/api/landmark/`,
            } 
    
            const response = await sendRequestAsync(config)
            return response.data
        }
        else {
            console.warn("[LandmarkData]: Can't create landmark. Given landmark value is null.")
        }
    }

    /**
     * The callback responsible for updating a {@link Landmark} on the server, used by the [react-query useMutation]{@link https://react-query.tanstack.com/reference/useMutation#_top} hook.
     * * @memberOf useLandmarks
     */
    const editLandmark =  async (landmarkValue: Landmark) => {
        if (landmarkValue) {
            const config: AxiosRequestConfig = {
                method: 'PUT',
                data: landmarkValue,
                url: `/api/landmark/`,
            } 

            const response = await sendRequestAsync(config)
            return response.data
        }
        else {
            console.warn("[LandmarkData]: Can't update landmark. Given landmark value is null.")
        }
    }

    /**
     * The callback responsible for updating a {@link Landmark} on the server, used by the [react-query useMutation]{@link https://react-query.tanstack.com/reference/useMutation#_top} hook.
     * * @memberOf useLandmarks
     */
     const rateLandmark =  async (data: {id: string, rating: 1 | -1}) => {
        const config: AxiosRequestConfig = {
            method: 'POST',
            data: data,
            url: `/api/landmark/rate/`,
        } 

        const response = await sendRequestAsync(config)
        return response.data
    }

    /**
     * The callback responsible for deleting a {@link Landmark} from the server, used by the [react-query useMutation]{@link https://react-query.tanstack.com/reference/useMutation#_top} hook.
     * * @memberOf useLandmarks
     */
    const removeLandmark =  async (id?: string | null) => {
        const config: AxiosRequestConfig = {
            method: 'DELETE',
            url: `/api/landmark/${id}`,
        } 
        const response = await sendRequestAsync(config)
        return response.data
    }

    // get-all query
    const { data: landmarks, status: getLandmarksStatus, refetch: refetchLandmarks } = useQuery<Landmark[], Error>('getLandmarks', () => getLandmarks(), {
        placeholderData: () => queryClient.getQueryData('getLandmarks'),
        retry: 3, 
        retryDelay: 3000,
        staleTime: 1000,
        refetchInterval: 30000,
        refetchOnReconnect: true,
        refetchOnMount: false
    })

    // get-details query
    const { data: landmark, status: getLandmarkStatus, refetch: refetchLandmark } = useQuery<Landmark, Error>(['getLandmark', options?.landmarkId], () => getLandmark(options?.landmarkId), {
        placeholderData: () => queryClient.getQueryData('getLandmark'),
        retry: 3, 
        retryDelay: 3000,
        refetchOnReconnect: true,
        refetchOnMount: false
    })

    const { data: landmarkRatedByUser, status: checkIfRatedByUserStatus, refetch: refetchCheckIfRatedByUser } = useQuery<boolean, Error>(['checkIfRatedByUser', options?.userLMPairing], () => checkIfRatedByUser(options?.userLMPairing), {
        placeholderData: () => queryClient.getQueryData('checkIfRatedByUser'),
        retry: 3, 
        retryDelay: 3000,
        refetchOnReconnect: true,
        refetchOnMount: false
    })

    // mutations
    const { status: addLandmarkStatus, mutateAsync: addLandmarkAsync, reset: resetAddLm, data: newLandmark } = useMutation(createLandmark, {
        onSuccess: data => {
            queryClient.invalidateQueries('getLandmarks')
        }, 
        retry: 3,
        retryDelay: 3000,
    })

    const { status: updateLandmarkStatus, mutateAsync: updateLandmark, reset: resetUpdateLm } = useMutation(editLandmark, {
        onSuccess: () => { queryClient.invalidateQueries('getLandmarks')},  
        onError: () => queryClient.invalidateQueries('getLandmarks'),  
        retry: 3,
        retryDelay: 3000,
    })

    const { data: rating, status: rateLandmarkStatus, mutateAsync: rateLandmarkAsync, reset: resetRateLandmark } = useMutation(rateLandmark, {
        onSuccess: () => { 
            queryClient.invalidateQueries('getLandmarks')
            queryClient.invalidateQueries('checkIfRatedByUser')
        },  
        onError: () => queryClient.invalidateQueries('getLandmarks'),  
        retry: 3,
        retryDelay: 3000,
    })

    const { status: deleteLandmarkStatus, mutateAsync: deleteLandmark, reset: resetDeleteLm } = useMutation(removeLandmark, {
        onSuccess: () => queryClient.invalidateQueries('getLandmarks'),  
        onError: () => queryClient.invalidateQueries('getLandmarks'),  
        retry: 3,
        retryDelay: 3000,
    })

    const { status: addPhotoStatus, mutateAsync: addPhoto, reset: resetAddPhoto } = useMutation(addLandmarkPhoto, {
        onSuccess: () => {
            queryClient.invalidateQueries('getLandmark')
        },
        
        onError: () => {
            queryClient.invalidateQueries('getLandmark')
        },
        retry: 3,
        retryDelay: 3000,
    })

    const { status: deletePhotoStatus, mutateAsync: deletePhoto, reset: resetDeletePhoto } = useMutation(deleteLandmarkPhoto, {
        onSuccess: () => {
            queryClient.invalidateQueries('getLandmark')
        },
        
        onError: () => {
            queryClient.invalidateQueries('getLandmark')
        },
        retry: 3,
        retryDelay: 3000,
    })

    return { 
        landmarks, getLandmarksStatus, refetchLandmarks,  //reading
        landmark, getLandmarkStatus, refetchLandmark, //photos
        addPhoto, addPhotoStatus, resetAddPhoto, //add photos
        deletePhoto, deletePhotoStatus, resetDeletePhoto, //delete photos
        addLandmarkAsync, resetAddLm, addLandmarkStatus, newLandmark, // creating
        updateLandmark, resetUpdateLm, updateLandmarkStatus, // updating
        deleteLandmark, resetDeleteLm, deleteLandmarkStatus, // deleting
        rateLandmarkAsync, resetRateLandmark, rateLandmarkStatus, rating, // rating
        landmarkRatedByUser, checkIfRatedByUserStatus, refetchCheckIfRatedByUser // check if landmark is rated by user
    }
}