import axios, { AxiosRequestConfig, AxiosResponse } from "axios";
import { loadAsync, ResponseType } from "expo-auth-session";
import { deleteItemAsync, getItemAsync, setItemAsync } from "expo-secure-store";
import jwtDecode from "jwt-decode";
import React, { createContext, useContext, useEffect, useMemo, useRef, useState } from "react";
import { Text, View } from "react-native";
import { MutationOptions, UseMutationOptions, useQueryClient, UseQueryOptions } from "react-query";
import { string } from "yup/lib/locale";
import { UserProfile } from "../../hooks/useProfile";
import { SECURESTORE_ACCESSTOKEN, SECURESTORE_PROFILE, SECURESTORE_REFRESHTOKEN } from "../../utils/GlobalUtils";
import { API_URL } from "../../utils/RequestUtils";
import { OIDC_DISCOVERY, OIDC_REDIRECT_URI } from "./config";
import { reportAxiosError } from "./helpers";

interface AuthContextData {
    accessToken?: string,
    refreshToken?: string,
    userProfile?: UserProfile,
    loading: boolean,
    logout: () => void,
    login: () => void,
    refreshAccessToken: (_refreshToken?: string) => Promise<{refreshToken: string, accessToken: string} | null>,
    sendRequestAsync: (config: AxiosRequestConfig, needsAuthentication?: boolean, retry?: boolean) => Promise<any>
}

interface ReactQueryRequestDetails {
    key: string
    type: "query" | "mutation"
    options: UseMutationOptions | UseQueryOptions
}

const AuthContext = createContext<AuthContextData | null>(null);

export const AuthProvider = ({children}) => {
    const [accessToken, setAccessToken] = useState<string>();
    const [refreshToken, setRefreshToken] = useState<string>();
    const [notificationToken, setNotificationToken] = useState<string>();
    const [userProfile, setUserProfile] = useState<UserProfile>();
    const [loading, setLoading] = useState<boolean>(false);

    const existingRefreshRequest = useRef<boolean>(false);
    const unresolvedQueries = useRef<ReactQueryRequestDetails[]>([])

    const queryClient = useQueryClient()

    useEffect(() => {
        const lookInSecureStoreForTokenOnMount = async () => {
            setLoading(true);
            
            setRefreshToken(await getItemAsync(SECURESTORE_REFRESHTOKEN))
            const accessTokenFromStore = await getItemAsync(SECURESTORE_ACCESSTOKEN)
            const userProfile = await getItemAsync(SECURESTORE_ACCESSTOKEN)

            if (accessTokenFromStore) {
                setAccessToken(accessTokenFromStore)
            }
            if (userProfile) {
                setUserProfile(JSON.parse(userProfile))
            }
        }
        lookInSecureStoreForTokenOnMount()
    }, []);

    useEffect(() => {
        const validateAccessTokenOnChange = async () => {
            if (accessToken) {
                console.log('token used by auth: ' +  accessToken)
                try {
                    await sendRequestAsync({
                        url: `/api/me`, 
                        headers: {Authorization: `Bearer ${accessToken}`}
                    });
                    await setItemAsync(SECURESTORE_ACCESSTOKEN, accessToken);
                    await setItemAsync(SECURESTORE_REFRESHTOKEN, refreshToken);

                    console.log("access token" + accessToken + " is valid")

                    await queryClient.resetQueries()

                    if (unresolvedQueries.current.length > 0) {
                        for (const query of unresolvedQueries.current) {
                            if (query.type == 'query') {
                                queryClient.fetchQuery(query.key, query.options)
                            }
                            if (query.type == 'mutation') {
                                queryClient.executeMutation(query.options)   
                            }
                        }
                    }

                } catch (error) {
                    console.error(error)
                    setAccessToken(undefined);
                    await deleteItemAsync(SECURESTORE_ACCESSTOKEN);
                    throw new Error("Invalid access token, clearing token from memory and store\n.")
                }
            }
            else {
                await deleteItemAsync(SECURESTORE_ACCESSTOKEN);
            }
        }
        validateAccessTokenOnChange()
    }, [accessToken]);
    useEffect(() => {
        const clearUserProfileFromStore = async () => {
            if (!userProfile) {
                await deleteItemAsync(SECURESTORE_PROFILE)
            }
        }
        clearUserProfileFromStore()
    }, [userProfile])

    /**
     * Function that initiates the login flow. It opens up with the in app browser and sends an authorization request to the API, which redirects the user to the backend login page. 
     * If the credentials entered are valid, the OAuth Authorization Code flow will occur, which results in the user recieving an access token and refresh token which are then stored in {@link AuthStore}.
     * */
    const login = async () => {
        setLoading(true)
        console.log('User is attempting to login, opening web portal login...')

        // initiate authentication request to the server 
        const request = await loadAsync({
            clientId: "atlas.mobile",
            responseType: ResponseType.Code,
            redirectUri: OIDC_REDIRECT_URI,
            usePKCE: true,
            scopes: ['openid'],
            
        }, OIDC_DISCOVERY)  

        // handle authentication response from the server
        const response = await request.promptAsync(OIDC_DISCOVERY);

        // if succesful, prepare a request for an access/id token
        if (response.type == "success" && request.codeVerifier) {
            console.log('User authentication was successful.')
            const tokenData = new URLSearchParams();
            tokenData.append('grant_type', 'authorization_code');
            tokenData.append('client_id', 'atlas.mobile');
            tokenData.append('code', response.params.code);
            tokenData.append('redirect_uri', request.redirectUri);
            tokenData.append('code_verifier', request.codeVerifier);  
            tokenData.append('notif-token', notificationToken);
            console.log('Attempting to retrieve access token...')

            // send the token request
            try {
                const response = await axios.post(API_URL + `/o/token/`, tokenData, {
                    headers: {
                        'Content-Type': "application/x-www-form-urlencoded"
                    },
                });  

                const tokenResponse = response.data;

                // if its a successful response, decode the jwt id token, and store the tokens in the corresponding stores
                const idToken = jwtDecode(tokenResponse.id_token) as {
                    sub: string, 
                    exp: number, 
                    iat: number, 
                    iss: string, 
                    nonce: string, 
                    email: string,
                    image_b64: string,
                    username: string,
                    show_tips: boolean,
                };

                console.log(JSON.stringify({...idToken, image_b64: idToken.image_b64.length}))

                // validate idtoken

                setAccessToken(tokenResponse?.access_token);
                setRefreshToken(tokenResponse?.refresh_token);
                setUserProfile({id: idToken.sub, username: idToken.username, email: idToken.email, image_b64: idToken.image_b64, show_tips: idToken.show_tips});
                setLoading(false)

                console.log('User successfully logged in.')
            } catch (error) {
                reportAxiosError("Something went wrong when retrieving access token", error);
            } 
        }
    }   

    const logout = async () => {
        setLoading(true);
        try {
            const tokenParams = new URLSearchParams();
            tokenParams.append('client_id', 'atlas.mobile');
            tokenParams.append('token', accessToken as string);
            
            const response = await axios.post(API_URL + `/o/revoke-token/`, tokenParams, {
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded'
                },
            });  

            setAccessToken(null);
            setRefreshToken(null);
            setNotificationToken(null);
            setUserProfile(null)
            setLoading(false);
        } catch (error) {
            reportAxiosError("Something went wrong when retrieving access token", error);
        } 
    }

    /**
         * If there is a refresh token available, attempts to use it to obtain a new, valid access token.
         * Used in {@link Atlas}
         * @memberOf useAuth
         */
     const refreshAccessToken = async (_refreshToken:string): Promise<{refreshToken: string, accessToken: string} | null> => {
        let refreshTokenForRequest = _refreshToken; 
        if (!refreshTokenForRequest) {
            refreshTokenForRequest = refreshToken;
        }
        if (!refreshTokenForRequest) {
            return null;
        }
        try {
            const tokenData = new URLSearchParams();
            tokenData.append('grant_type', 'refresh_token');
            tokenData.append('refresh_token', refreshToken);
            tokenData.append('client_id', 'atlas.mobile');
            console.log('Attempting to refresh token...')

            const response = await axios.post(API_URL + "/o/token/", tokenData, {
                headers: { 'Content-Type': "application/x-www-form-urlencoded" }
            });   

            console.log(response.data)

            console.log('Token successfully refreshed.')
            setRefreshToken(response?.data?.refresh_token);
            setAccessToken(response?.data?.access_token);
        }
        catch (error) {
            reportAxiosError("Error when trying to refresh access token", error);            
            return null
        }   
        return null
    }

    const cnpAxios = axios.create({
        baseURL: API_URL
    })

    const sendRequestAsync = async (config: AxiosRequestConfig, needsAuthentication: boolean = true, retry: boolean = true, queryDetails?: ReactQueryRequestDetails): Promise<any> => {
        if (needsAuthentication) {
            if (accessToken) {
                config.headers = {...config.headers, Authorization: `Bearer ${accessToken}`};
                console.log("config: " + config)
            }
            else {
                throw new Error("No valid access token, canceling request\n. Request details: " + JSON.stringify(config))
            }
        }

        if (!accessToken && needsAuthentication) {
            throw new Error("No valid access token, canceling request\n. Request details: " + JSON.stringify(config))
        }

        try {
            const response = await cnpAxios(config);
            console.log("successfully authorized call to " + config.url + " with access token: " + accessToken)
            return response;
        } catch (error) {
            if (error.response.status == 401 && retry && accessToken) {
                if (!existingRefreshRequest.current) {
                    console.log("access token " + accessToken + " for call to " + config.url + " was not authorized")
                    existingRefreshRequest.current = true
                    try {
                        const refreshData = await refreshAccessToken(refreshToken);
                        existingRefreshRequest.current = false;
                    } catch (error) {console.error(error)}
                }   
                
                unresolvedQueries.current = [...unresolvedQueries.current, queryDetails];
                throw error
            }
            else {
                console.error("Request failed: " + JSON.stringify(error?.response?.config))
            }
        }
    }

    const authState = useMemo<AuthContextData>(
        () => ({
            loading,
            accessToken,
            refreshToken,
            login,
            logout,
            refreshAccessToken,
            sendRequestAsync,
            userProfile
        }),
        [loading, userProfile, accessToken, refreshToken]
    );

    return (
        <AuthContext.Provider value={authState}>
            {children}
        </AuthContext.Provider>
    )
}

export const useAuth = () => {
    const context = useContext(AuthContext);
    if (!context) {
        throw new Error('useAuth must be used within a AuthProvider');
    }
    return context;
}
