import React, { createContext, useContext, useEffect, useMemo, useRef, useState } from "react";
import { AxiosRequestConfig } from "axios";
import Constants from "expo-constants";
import { deleteItemAsync, getItemAsync, setItemAsync } from "expo-secure-store";
import { SECURESTORE_NOTIFTOKEN, SECURESTORE_REFRESHTOKEN } from "../../utils/GlobalUtils";
import { useAuth } from "./AuthContext";
import * as Notifications from 'expo-notifications'
import { Platform } from "react-native";
import { useProfile } from "../../hooks/useProfile";

const NotifContext = createContext<string>('');

export const NotifProvider: React.FC = ({children}) => {
    const [notificationToken, setNotificationToken] = useState<string>();

    const {userId, accessToken, refreshAccessToken} = useAuth()
    const previousUserId = useRef<string>(userId)
    const {refetchNotifications} = useProfile(userId)

    const {sendRequestAsync} = useAuth()

    useEffect(() => {
        previousUserId.current = userId
    }, [userId])

    useEffect(() => {
        const retrieveNotificationToken = async () => {
            // this signals a new user, so we need to change the notification token
            if (previousUserId?.current && userId != previousUserId.current) {
                await getNotificationTokenFromServer()
            }
            // otherwise, check the stored notification token
            else {
                let currentToken = notificationToken
                if (!currentToken) {
                    currentToken = await getItemAsync(SECURESTORE_REFRESHTOKEN);   
                }
                if (currentToken) {
                    await ensureNotificationTokenExistsOnServer(currentToken, 5)
                    setNotificationToken(currentToken)
                }
            }
        }
        retrieveNotificationToken()
    }, [userId]);

    useEffect(() => {
        const handleNotifTokenChange = async () => {
            if (notificationToken) {
                await registerForPushNotificationsAsync()
            }
            else {
                await deleteItemAsync(SECURESTORE_NOTIFTOKEN)
            }
        }
        handleNotifTokenChange()
    }, [notificationToken])

    const getNotificationTokenFromServer = async () => {
        const config: AxiosRequestConfig = {
            method: 'GET',
            url: `/api/notif-token/${userId}/`,
        }
        
        try {
            const response = await sendRequestAsync(config)
            const token = response?.data?.token
            await setItemAsync(SECURESTORE_NOTIFTOKEN, token)
            setNotificationToken(token)
        }
        catch (error) {
            throw error
        }
    }

    const ensureNotificationTokenExistsOnServer = async (notificationToken: string, retries: number) => {
        try {
            const config: AxiosRequestConfig = {
                method: 'POST',
                data: {token: notificationToken},
                url: `/api/notif-token/${userId}/`,
            } 
            await sendRequestAsync(config)
        }
        catch (error) {
            if (retries > 1) {
                await ensureNotificationTokenExistsOnServer(notificationToken, retries - 1)   
            }
            else throw error
        }
    }

    const registerForPushNotificationsAsync = async () => {
        if (Constants.isDevice) {
            const { status: existingStatus } = await Notifications.getPermissionsAsync();
            let finalStatus = existingStatus;
            if (existingStatus !== 'granted') {
                const { status } = await Notifications.requestPermissionsAsync();
                finalStatus = status;
            }
            if (finalStatus !== 'granted') {
                alert('Failed to get push token for push notification!');
                setNotificationToken('')
                return;
            }

            if (Platform.OS === 'android') {
                Notifications.setNotificationChannelAsync('default', {
                    name: 'default',
                    importance: Notifications.AndroidImportance.MAX,
                    vibrationPattern: [0, 250, 250, 250],
                    lightColor: '#FF231F7C',
                });
            }   
        }
    };

    const notifToken = useMemo<string>(
        () => notificationToken, [notificationToken]
    );

    return (
        <NotifContext.Provider value={notifToken}>
            {children}
        </NotifContext.Provider>
    )
}

export const useNotifications = () => {
    const notifToken = useContext(NotifContext);
    return notifToken;
}
